@@ -18,7 +18,7 @@ our @EXPORT_OK = qw( cldr_format cldr_parse );
 
 # CPAN data
 our $AUTHORITY = 'cpan:MAROS';
-our $VERSION = '1.16';
+our $VERSION = '1.17';
 
 # Default format if none is set
 our $DEFAULT_FORMAT = 'date_format_medium';
@@ -164,7 +164,9 @@ our %PARSER = (
     G5      => 'era_narrow',
     y1      => $PARTS{year_long},
     y2      => $PARTS{year_short},
-    y3      => $PARTS{year_long},
+    y3      => qr/(-?\d{3,4})/o,
+    y4      => qr/(-?\d{4})/o,
+    y5      => qr/(-?\d{5})/o,
     Y1      => $PARTS{year_long},
     u1      => $PARTS{year_long},
     Q1      => $PARTS{quarter},
@@ -205,7 +207,7 @@ our %PARSER = (
    #j1      => Handled dyamically,
     m1      => $PARTS{minute},
     s1      => $PARTS{second},
-    S1      => $PARTS{number},
+    S1      => $PARTS{number}, #!
     Z1      => [ grep { $_ ne 'Ambiguous' } values %ZONEMAP ],
     Z4      => $PARTS{timezone2},
     z1      => [ keys %ZONEMAP ],
@@ -436,7 +438,7 @@ sub locale {
     # Set locale
     if (defined $locale) {
         unless (ref $locale
-            && $locale->isa('DateTime::Locale::Base')) {
+            && ($locale->isa('DateTime::Locale::Base') || $locale->isa('DateTime::Locale::FromData'))) {
             $self->{locale} = DateTime::Locale->load( $locale )
                 or croak("Could not create locale from $locale");
         } else {
